# *************************************************
#
# USAGE
# 
# *************************************************
#
# Define Airbox.site & Airbox.auth_token
# (Automatically scans for init.rb in same dir)
#
# *************************************************
#
# Resources are paginated. You get 30 at a time.
#
# @resources = Airbox::Group.find :all
# @resources = @resources.first.notices(:page => 2)
# @resources = Airbox::Error.all(show_resolved: true)
#
# *************************************************

require "active_resource"

module Airbox
  # Accessor methods for site/auth_token settings
  # H/T to Instagram gem team for the inspiration
  def self.site
    Resource.site
  end

  def self.site=(site)
    Resource.site = site
  end

  def self.auth_token
    Resource.auth_token
  end

  def self.auth_token=(tok)
    Resource.auth_token=(tok)
  end

  class Resource < ::ActiveResource::Base
    # Airbrake's API responds with XML objects
    self.format = ActiveResource::Formats::XmlFormat

    class << self
      # Store auth_token as class variable
      def auth_token=(tok)
        @@auth_token = tok
      end

      # Before standard ActiveResource::Base.find, prep params for request
      def find(*arguments)
        if !site
          err = "No site defined. Please run Airbox.site = 'site_url'"
          raise ConfigError, err
        end
        arguments = prepare_params(*arguments)
        super(*arguments)
      end

      # Move opts (except for "from") into params hash
      # Add auth_token to params hash
      def prepare_params(*arguments)
        if !class_variable_defined?(:@@auth_token)
          err = "No auth token. Please run Airbox.auth_token = 'token'"
          raise ConfigError, err
        end

        opts = arguments.last.is_a?(Hash) ? arguments.pop : {}

        # Store pre-transformed opts in instance var
        # Later accessed to ensure continuity on paged results
        @opts = opts.dup

        opts[:params] = add_relevant_opts_to_params(opts.except(:from))
        opts[:params].merge!(auth_token: @@auth_token)
        arguments << opts
        arguments
      end

      # Method to move opts into params hash
      def add_relevant_opts_to_params(opts)
        params = opts[:params] || {}
        opts.except(:params).each do |k,v|
          params[k] = v
        end
        params
      end

      # After standard ActiveResource::Base.instantiate_collection,
      # ensure collection is returned as Airbox::Array for easier use
      #
      # TODO - needs update to ActiveResource4 + backwards compatibility
      # (Currently conforms to ActiveResource 3.2.13 spec)
      def instantiate_collection(collection, original_params = {})
        ary = super(collection, original_params)
        ary = Array.new(ary)
        ary.opts = @opts
        ary
      end

      def ensure_collection(coll)
        klass = self.class.to_s.split('::').last.downcase
        if coll.has_key?(klass)
          coll = coll[klass]
        end
        coll
      end
    end
  end

  # Airbrake project resources
  class Project < Resource
    # For some reason, these return as JSON?
    self.format = ActiveResource::Formats::JsonFormat

    # Return project's errors
    def errors(opts={})
      Error.all(opts.merge(project_id: id))
    end
  end

  # Airbrake error resources
  class Error < Resource

    # Collect all associated data
    def full_obj(opts={})
      self.class.find(id, opts)
    end

    # Return related group
    def group(opts={})
      Group.find(id)
    end
  end

  # Airbrake group resources
  class Group < Resource

    # Return group's notices
    def notices(opts={})
       Notice.all(opts.merge(from: "/groups/#{id}/notices"))
    end

    # Airbrake notice resources
    class Notice < Resource

      # Collect all associated data
      def full_obj(opts={})
        self.class.find :one, opts.merge(from: "/groups/#{group_id}/notices/#{id}")
      end
    end
  end

  # Extend Array class for module use
  # When Airbox returns collections of resources, give them paging methods
  class Array < ::Array
    # Store/retrieve query in opts instance variable
    attr_accessor :opts

    # Return next page of results for current query
    def next_page
      @opts ||= {}
      @opts[:page] ||= 1
      return first.class.all(@opts.merge(page: @opts[:page] + 1).dup)
    end

    # Return all future pages of results for current query
    def next_pages
      collection = []
      current = self
      while(current.count == 30)
        current = current.next_page
        collection += current
      end
      collection
    end

    # Return all pages of results for current query
    def all_pages
      if !self.opts || self.opts[:page] == 1 || !self.opts[:page]
        self + self.next_pages
      else
        collection = first.class.all(@opts.except(:page))
        collection + collection.next_pages
      end
    end
  end

  class ConfigError < ::StandardError
  end
end

# Load init file in same folder
# Rescue if no initialization found
begin
  require "./airbox"
rescue LoadError
end

